@@ -1,6 +1,6 @@
 #
 #
-# Copyright (c) 2021 DPS, dps@my.mail.de
+# Copyright (c) 2020-2021 DPS, dps@my.mail.de
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,11 +16,10 @@
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 #
 #
-from Dhelpers.all import launch
-from Dhelpers.adaptor import DependencyManager
+from .. import launch, DependencyManager
 
-with DependencyManager(__name__) as tester:
-    tester.test_shellcommand("xsel", pkg="xsel")
+with DependencyManager(__name__) as manager:
+    manager.test_shellcommand("xsel", pkg="xsel")
 
 xsel_names = {0:"b",1:"p",2:"s"}
 
