@@ -134,5 +134,6 @@ class playlist_creator(FileSelector):
     file_start = "#EXTM3U"
     
     @staticmethod
-    def default_selection_func(obj,genre=None):
-        return genre in obj.genre
\ No newline at end of file
+    def default_selection_func(obj,genre=None,case_sensitive=False):
+        if case_sensitive: return genre in obj.genre
+        return genre.lower() in obj.genre.lower()
\ No newline at end of file
