@@ -23,6 +23,7 @@ from Dhelpers.arghandling import check_type
 from abc import ABC, abstractmethod
 import threading, queue, logging, inspect, functools, traceback
 from ..event_classes import MouseMoveEvent
+from Dhelpers.named import NamedObj
 
 class CallbackRunnerThread(threading.Thread):
     pass
@@ -47,7 +48,6 @@ class EventQueueReader:
     def run_callbacks(self):
         while True:
             handler, event_obj = self.event_queue.get()
-            # cls is a subclass of CallbackHook
             handler.run_callbacks(event_obj)
 
 
@@ -55,58 +55,72 @@ class EventQueueReader:
 
 class InputEventHandler(ABC):
     
+    # base class to create Capturer and Collector classes
+    
     reinject_implemented = False
+    capture_allowed = True  #need to be manually activated
+    collect_allowed = True  #need to be manually activated
     
     def __init__(self, hook_cls=None):
-        self.hook_cls = hook_cls
-        self.active = False
-        self.hook_num = 0
+        self.hook_cls = hook_cls  #this attribute can be left None as it will
+        # usually be defined by __init_subclass__ method of CallbackHook class
+        self.collect_active = False
+        self.capture_active = False
         self.active_hooks = []
+        self.cap_num = 0
 
-    def add_one(self):
-        self.hook_num += 1
-        if self.hook_num == 1:
-            self.start()
-            return True
 
-    def remove_one(self):
-        self.hook_num -= 1
-        if self.hook_num == 0:
-            self.stop()
-            return True
-        
-        
     def add_hook(self, hook):
-        self.active_hooks.append(hook)
-        self.active_hooks.sort(key=lambda s: s.priority)
-        self.add_one()
+        ah = self.active_hooks
+        ah.append(hook)
+        ah.sort(key=lambda s: s.priority)
+        if len(ah) == 1:
+            if self.collect_active: raise RuntimeError
+            self.collect_active = True
+            self.start_collecting()
+            return True
 
     def remove_hook(self, hook):
         self.active_hooks.remove(hook)
-        self.remove_one()
-        
-        
-    def start(self):
-        if self.active: raise RuntimeError
-        self.active = True
-        #self.prepare()
-        self.run()
-        
-    def stop(self):
-        if not self.active: raise RuntimeError
-        self.active = False
-        self.terminate()
-            
-    @abstractmethod
-    def run(self):
-        # must be non_blocking
-        pass
-    
-    @abstractmethod
-    def terminate(self):
-        pass
+        if len(self.active_hooks) == 0:
+            if not self.collect_active: raise RuntimeError
+            self.collect_active = False
+            self.stop_collecting()
+            return True
     
+    def add_capturer(self):
+        self.cap_num += 1
+        if self.cap_num == 1:
+            if self.capture_active: raise RuntimeError
+            self.capture_active = True
+            self.start_capturing()
+            return True
 
+    def remove_capturer(self):
+        self.cap_num -= 1
+        if self.cap_num == 0:
+            if not self.capture_active: raise RuntimeError
+            self.capture_active = False
+            self.stop_capturing()
+            return True
+        
+        
+    def start_collecting(self):
+        # must be non_blocking
+        raise NotImplementedError
+        
+    def stop_collecting(self):
+        pass
+
+
+    def start_capturing(self):
+        # must be non_blocking
+        raise NotImplementedError
+
+    def stop_capturing(self):
+        pass
+        
+    
     def queue_event(self, *args, **kwargs):
         try:
             cls = self.hook_cls
@@ -126,12 +140,14 @@ class InputEventHandler(ABC):
             except Exception:
                 pass
             try:
-                self.stop()
+                self.stop_collecting()
+            except Exception:
+                pass
+            try:
+                self.stop_capturing()
             except Exception:
                 pass
             traceback.print_exc()
-        
-
     
     def run_callbacks(self, event_obj):
         if event_obj is None: return
@@ -139,13 +155,29 @@ class InputEventHandler(ABC):
             ret = hook.run_callback(event_obj)
             if ret == "block": break
             
+            
+    
+    
+
+class Missing: pass
+    
     
 class CallbackHook(AdditionContainer.Addend, TimedObject):
     
-    #these can be overwritten by instance attributes if they depend on
-    # instance arguments. However, usually they are set by __init_subclass__
-    capturer = None
-    collector = None
+    # usually set by __init_subclass__ or subclass attribute. In these cases,
+    # there is at most one handler instance for each subclass.
+    # They can optionally be set for a particular Hook instance, in which
+    # case this instance has it's own handler available (CustomHook subclass
+    # uses this for evdev)
+    handler = None
+    
+    @property
+    def capture_allowed(self):
+        return self.handler.capture_allowed
+        
+    @property
+    def collect_allowed(self):
+        return self.handler.collect_allowed
     
     
     queue_reader = EventQueueReader()
@@ -158,19 +190,25 @@ class CallbackHook(AdditionContainer.Addend, TimedObject):
         cls._active_capturers = 0
         cls._active_reinject_func = None
         cls.reinject_active = False
-        for handler in (cls.collector, cls.capturer):
-            if handler is not None:
-                check_type(InputEventHandler, handler)
-                if handler.hook_cls is None: handler.hook_cls = cls
-                if handler.hook_cls is not cls: raise ValueError
+        handler = cls.handler
+        if handler is not None:
+            check_type(InputEventHandler, handler)
+            if handler.hook_cls is None: handler.hook_cls = cls
+            if handler.hook_cls is not cls: raise ValueError
         cls.__init__.__signature__ = inspect.signature(cls.init)
 
 
 
     def __init__(self, *args, **kwargs):
-        self._init_args = args
-        self._init_kwargs = kwargs
-        self.init(*args, **kwargs)
+        self._init_kwargs = self._swallow_args(*args,**kwargs)
+        self.init(**self._init_kwargs)
+    
+    @staticmethod
+    def _swallow_args(callback_func=Missing,timeout=Missing, **kwargs):
+        # btw: makes sure that no positional arg is also specified in kwargs
+        if callback_func is not Missing: kwargs["callback_func"]=callback_func
+        if timeout is not Missing: kwargs["timeout"] = timeout
+        return kwargs
     
     def init(self, callback_func = False, timeout = 60, *, capture: bool =
             False, reinject_func = None, priority: int  = 0,
@@ -189,12 +227,10 @@ class CallbackHook(AdditionContainer.Addend, TimedObject):
         self.capture = capture
         self.custom_kwargs = custom_kwargs
         self.dedicated_thread = dedicated_thread
-        if custom_kwargs:
-            self.process_custom_kwargs(**custom_kwargs)
-            # for the evdev custom hook, this might set the self.capturer and
-            # self.collector variables
+        self.process_custom_kwargs(**custom_kwargs)
+        if self.callback_func and not self.collect_allowed: raise ValueError
         if reinject_func is not None:
-            if self.collector.reinject_implemented is False:
+            if self.handler.reinject_implemented is False:
                 raise NotImplementedError(
                     "The following hook implementation does not support "
                     "reinjecting:\n" + str(self.__class__))
@@ -203,33 +239,32 @@ class CallbackHook(AdditionContainer.Addend, TimedObject):
             self.capture = True
             if not callable(reinject_func):
                 self.reinject_func = lambda *args: reinject_func
-        if not self.capturer and self.capture is True:
+        if self.capture is True and not self.capture_allowed:
             raise NotImplementedError(
                     "The following hook implementation does not support "
                     "capturing:\n" + str(self.__class__))
-
-        
-    def __call__(self, *args, reuse=True, **kwargs):
+    
+    def __call__(self, *args, _reuse=True, **kwargs):
         """Create a copy, Adding left out arguments.
         For a useful example of successively adding argument see
         Dpowers.events.waiter.Keywaiter class."""
-        if not reuse: return self.__class__(*args,**kwargs)
-        newargs = list(args)
-        for i in range(len(args), len(self._init_args)):
-            # this happens if less new args were given than inital args,
-            # so we will reuse the old args as much as possible
-            newargs.append(self._init_args[i])
+        if not _reuse: return self.__class__(*args,**kwargs)
         newkwargs = self._init_kwargs.copy()
-        newkwargs.update(kwargs)
-        return self.__class__(*newargs, **newkwargs)
+            #this includes initial *args and **kwargs values
+        newkwargs.update(self._swallow_args(*args, **kwargs))
+            #this will overwrite inital *args and **kwargs value
+        return self.__class__(**newkwargs)
 
     def __bool__(self):
         return self.callback_func is not False or self.capture is True
 
         
     def process_custom_kwargs(self, **kwargs):
-        raise NotImplementedError(f"Hook {self} does not accept custom kwargs {kwargs}")
+        if not kwargs: return
+        raise NotImplementedError(f"Hook {self} does not accept custom kwargs.")
 
+
+    # this method is called from cls.collector instance to run the callbacks
     def run_callback(self, event_obj):
         if self.callback_func is None: return
         event_obj = self.callback_condition(event_obj)
@@ -258,11 +293,11 @@ class CallbackHook(AdditionContainer.Addend, TimedObject):
         if self.capture:
             cls._active_capturers += 1
             cls._update_reinject()
-            self.capturer.add_one()
+            self.handler.add_capturer()
         if self.callback_func is not False:
             self.queue_reader.start()  # just in case the queue_reader has
             # somehow been stopped, lets recheck everytime
-            self.collector.add_hook(self)
+            self.handler.add_hook(self)
 
     # stop method is defined in TimedObj Baseclass and will use this:
     def _stop_action(self):
@@ -272,9 +307,9 @@ class CallbackHook(AdditionContainer.Addend, TimedObject):
         if self.capture:
             cls._active_capturers -= 1
             cls._update_reinject()
-            self.capturer.remove_one()
+            self.handler.remove_capturer()
         if self.callback_func is not False:
-            self.collector.remove_hook(self)
+            self.handler.remove_hook(self)
     
     def _timeout_action(self):
         warn("Timeout after %s seconds: Stopping hook %s." % (
@@ -314,6 +349,8 @@ class CallbackHook(AdditionContainer.Addend, TimedObject):
     #         if ret == "stop": self.stop()
 
 
+    # the following method is called by the cls.collector instance to create
+    # event objects.
     @classmethod
     @abstractmethod
     def _create_event_obj(cls, *args, **kwargs):
@@ -321,51 +358,37 @@ class CallbackHook(AdditionContainer.Addend, TimedObject):
 
 
     def callback_condition(self, event_obj):
-        # can be overridden in subclass. event_obj will be adapted for
-        # specific instance
+        # can be overridden in subclass.
         return event_obj
-    
 
 
-
-
-class PressReleaseHook(CallbackHook):
-    
-    NamedClass = None # set by target modifier of HookAdaptor
-    
-    
-    #needs to be overwritten in subclass:
-    currently_pressed = None
+class NamedHook(CallbackHook):
+    NamedClass = None   #set by target_modifier of HookAdaptor
+    EventClass = None
     name_translation_dicts = None
-    _active_dict = None
+    #name_translation_dict = None
+    _active_dict = {}
+    
+    
     
-
     def __init_subclass__(cls):
         super().__init_subclass__()
-        cls.currently_pressed = set()
         if hasattr(cls, "name_translation_dict"):
             if len(cls.name_translation_dicts) != 0: raise AttributeError
             cls.name_translation_dicts = [cls.name_translation_dict]
         else:
             cls.name_translation_dicts = []
 
-    def init(self, *args, press=True, release = True, write_rls=True, **kwargs):
-        CallbackHook.init(self,*args, **kwargs)
-        self.press = press
-        self.release = release
-        self.multipress = True
-        self.write_rls=write_rls
-        
     @classmethod
     def update_active_dict(cls):
-        #this has to be called after cls.NamedClass has been set
+        # this has to be called after cls.NamedClass has been set
         for i in range(len(cls.name_translation_dicts)):
             dic = cls.name_translation_dicts[i]
             StandardizingDict = cls.NamedClass.StandardizingDict
             if not isinstance(dic, StandardizingDict):
-                check_type(dict,dic)
+                check_type(dict, dic)
                 cls.name_translation_dicts[i] = StandardizingDict(dic)
-        coll = cls.collector
+        coll = cls.handler
         if coll is None: return
         check_type(InputEventHandler, coll)
         try:
@@ -375,7 +398,7 @@ class PressReleaseHook(CallbackHook):
         if hasattr(coll, "names"):
             for n in coll.names:
                 if n not in final_dict: final_dict[n] = n
-
+    
         for a, b in final_dict.items():
             for dic in cls.name_translation_dicts:
                 try:
@@ -386,34 +409,66 @@ class PressReleaseHook(CallbackHook):
                 final_dict[a] = cls.NamedClass.instance(b)
             except KeyError:
                 final_dict[a] = b
-        cls._active_dict = final_dict #this is used each time a key is pressed. It
-                 # is not standardizing to save time.
+        cls._active_dict = final_dict  # this is used each time a key is
+        # pressed. It  # is not standardizing to save time.
         
         
+    @classmethod
+    def _create_event_obj(cls, name, **kwargs):
+        effective = cls._active_dict.get(name, name)
+        NamedCl = cls.NamedClass
+        if NamedCl is None:
+            return cls.EventClass(effective, **kwargs)
+        else:
+            try:
+                EventCl = NamedCl.Event
+            except AttributeError:
+                EventCl = cls.EventClass
+            else:
+                if cls.EventClass is not None: assert cls.EventClass is EventCl
+                if isinstance(effective, NamedCl) and hasattr(effective,
+                        "get_event"):
+                    return effective.get_event(**kwargs)
+            event_obj = EventCl(effective, **kwargs)
+            # print(event_obj, event_obj.named_instance, event_obj.press)
+            if not event_obj.named_instance:
+                if not (event_obj.startswith("[")):
+                    warn("hook of type" + str(
+                            cls) + "got unknown key/button name:  " + event_obj)
+                    if not event_obj.endswith("_rls"):
+                        i = event_obj.lower()
+                        container.set_temp_store_key("F12_send",
+                                i + " = NamedKey('" + i + "')")
+            return event_obj
+
+
+
+class PressReleaseHook(NamedHook):
+    
+    currently_pressed = None
+    
+    def __init_subclass__(cls):
+        super().__init_subclass__()
+        cls.currently_pressed = set()
+        
+
+    def init(self, *args, press=True, release = True, write_rls=True, **kwargs):
+        super().init(*args, **kwargs)
+        self.press = press
+        self.release = release
+        self.multipress = True
+        self.write_rls=write_rls
+        
+    
     def _stop_action(self):
         super()._stop_action()
-        if not self.collector.active_hooks:
+        if not self.handler.active_hooks:
             self.__class__.currently_pressed = set()
 
 
     @classmethod
     def _create_event_obj(cls, name, press, **kwargs):
-        effective = cls._active_dict.get(name,name)
-        if isinstance(effective, cls.NamedClass):
-            event_obj = effective.get_event(press, **kwargs)
-        else:
-            event_obj = cls.NamedClass.Event(effective, press=press,
-                    **kwargs)
-        #print(event_obj, event_obj.named_instance, event_obj.press)
-        if not event_obj.named_instance:
-            if not (event_obj.startswith("[")):
-                warn("hook of type" + str(
-                        cls) + "got unknown key/button name:  " + event_obj)
-                if not event_obj.endswith("_rls"):
-                    i = event_obj.lower()
-                    container.set_temp_store_key("F12_send",
-                            i + " = NamedKey('" + i + "')")
-        return event_obj
+        return super()._create_event_obj(name,press=press, **kwargs)
     
     
     def callback_condition(self, event_obj):
@@ -433,7 +488,7 @@ class KeyhookBase(PressReleaseHook):
     
     
     def init(self, *args, allow_multipress=False, **kwargs):
-        PressReleaseHook.init(self,*args, **kwargs)
+        super().init(*args, **kwargs)
         if allow_multipress and not self.press: raise ValueError
         self.multipress = allow_multipress
         
@@ -448,8 +503,11 @@ class KeyhookBase(PressReleaseHook):
             else:
                 event_obj.multipress = False
                 cls.currently_pressed.add(info)
-        elif event_obj.press is False:
-            cls.currently_pressed.remove(info)
+        else:
+            try:
+                cls.currently_pressed.remove(info)
+            except KeyError:
+                pass
         return event_obj
 
 
@@ -463,7 +521,7 @@ class CursorhookBase(CallbackHook):
         return MouseMoveEvent(*args, **kwargs)
 
 
-class CustomhookBase(CallbackHook):
+class DefaulthookBase(CallbackHook):
     
     @classmethod
     def _create_event_obj(cls, *args):
