@@ -27,6 +27,11 @@ global_uinput = uinput.global_uinput
 from evdev.ecodes import KEY, EV_KEY
 
 names = {}
+backend_layout = "evdev"
+layout = "us"
+use_shifted = True
+
+
 
 def iterator(dic):
     for a, b in dic.items():
@@ -37,7 +42,7 @@ def iterator(dic):
 
 for integer, name in iterator(KEY):
     n = name[4:]
-    if len(n)==1 and n.isalpha(): n=n.lower()
+    n=n.lower()
     names[n] = integer
     
 global_uinput.start()
