@@ -1,6 +1,6 @@
 #
 #
-# Copyright (c) 2021 DPS, dps@my.mail.de
+# Copyright (c) 2020-2021 DPS, dps@my.mail.de
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,18 +17,18 @@
 #
 #
 import shlex, re
-from Dhelpers.all import (CollectionWithProps, NonNegativeInt, launch)
+from Dhelpers.arghandling import CollectionWithProps, NonNegativeInt
 
-from Dhelpers.adaptor import DependencyManager
+from .. import DependencyManager, launch
 
-with DependencyManager(__name__) as tester:
-    current_id = tester.test_shellcommand("xdotool",test_cmd = "xdotool "
+with DependencyManager(__name__) as manager:
+    current_id = manager.test_shellcommand("xdotool",test_cmd ="xdotool "
                                    "getactivewindow", pkg="xdotool")
     #checks if xdotool is installed and if getactivewindow works
 
-    tester.test_shellcommand("wmctrl", pkg="wmctrl")
+    manager.test_shellcommand("wmctrl", pkg="wmctrl")
 
-    tester.test_shellcommand("xprop",f"xprop -id {current_id} WM_CLASS")
+    manager.test_shellcommand("xprop", f"xprop -id {current_id} WM_CLASS")
     # checks if xprop is installed and if it gets the current window's class
 
 
@@ -103,8 +103,8 @@ def _xdo_get_from_single_id(ID, add_final):
 # find all window ids matching the given pattern
 
 def _xdo_get_ids(*win_args, **win_kwargs):
-    cmd = _xdo_cmd(*win_args, **win_kwargs,
-            add_final=None)  # add_final=None makes sure that IDs are returned
+    cmd = _xdo_cmd(*win_args,  **win_kwargs, add_final=None)
+        # add_final=None makes sure that IDs are returned
     try:
         x = launch.get(cmd)
     except launch.CalledProcessError:
@@ -134,20 +134,20 @@ def ID_from_location(param):
 
 allowed_properties = "title", "wcls", "pid"
 
-def IDs_from_property(prop_name, prop_val):
+def IDs_from_property(prop_name, prop_val, visible):
     if prop_name == "title":
-        return _xdo_get_ids(title=prop_val)
+        return _xdo_get_ids(title=prop_val, onlyvisible=visible)
     elif prop_name == "wcls":
         wcls = prop_val
         if "." in wcls:
             cl_name, cl = wcls.split(".")
             # cls is a string of pattern "cl_name.cl" usually
-            return tuple(set(_xdo_get_ids(cl=cl)) & set(
-                    _xdo_get_ids(cl_name=cl_name)))
+            return tuple(set(_xdo_get_ids(cl=cl, onlyvisible=visible)) & set(
+                    _xdo_get_ids(cl_name=cl_name, onlyvisible=visible)))
         else:
-            return _xdo_get_ids(cl=wcls)
+            return _xdo_get_ids(cl=wcls, onlyvisible=visible)
     elif prop_name == "pid":
-        return _xdo_get_ids(pid=prop_val)
+        return _xdo_get_ids(pid=prop_val, onlyvisible=visible)
     raise NotImplementedError
 
 
@@ -158,7 +158,9 @@ def property_from_ID(ID, prop_name):
         x = re.split('"', launch.get("xprop", "-id", ID, "WM_CLASS"))
         return x[-4] + '.' + x[-2]
     elif prop_name == "pid":
-        return int(_xdo_get_from_single_id(ID, "getwindowpid"))
+        p = _xdo_get_from_single_id(ID, "getwindowpid")
+        if p is None: return
+        return int(p)
     elif prop_name == "geometry":
         g = _xdo_get_from_single_id(ID, "getwindowgeometry -shell")
         x = re.split("[\n=”]", g)
@@ -170,6 +172,11 @@ def property_from_ID(ID, prop_name):
 def activate(ID):
     return _xdo_get_from_single_id(ID, "windowactivate --sync")
 
+def map(ID):
+    return _xdo_get_from_single_id(ID,"windowmap --sync")
+
+def unmap(ID):
+    return _xdo_get_from_single_id(ID,"windowunmap --sync")
 
 def set_prop(ID, action: str, prop: str, prop2):
     cmd = ["wmctrl", "-i", "-r", ID, "-b", action + "," + prop]
